#
# ksdb.py - like treedb.py, functions for handling a database of kickstarts
#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#

import sys
import os
import glob
import imputil
from snake.config import getoption
from snake.kickstart import Kickstart,PyKickstart

import logging
log = logging.getLogger("snake")

def add(newfile,name=None,desc=None):
    '''Add the given file (or open file-like object) to the kickstart database.
    if the newfile is an open file-like object you can pass the 'name' param
    to give it a name.'''
    if hasattr(newfile,'read'):
        fp = newfile
        if not name:
            raise ValueError, "You need a name with a file-like object, yo."
    else:
        fp = open(newfile)
        if not name:
            (name,ext) = os.path.splitext(os.path.basename(newfile))

    write_data(fp.read(),name,desc)
    return True # XML-RPC requires a return value

def write_data(data,name,desc=None):
    '''Write the given kickstart data (character buffer) to an entry in the
    ksdb named 'name'.'''
    ksdb_dir = getoption("ksdb_dir")
    template_dir = getoption("templatedir")
    k = None

    # validate kickstart - caller responsible for exception handling
    try:
        # first try to read the kickstart as if it were python cod3
        k = PyKickstart()
        k.readbuf(data)
        ksfile = os.path.join(template_dir,name+'.py')
    except Exception, e:
        # interpretting as python failed, howabout a plain txt kickstart
        k = Kickstart()
        k.readbuf(data)
        ksfile = os.path.join(ksdb_dir,name+'.ks')
        pass

    if k != None:
        if desc: k.description = desc
        out = open(ksfile, 'w')
        k.writefp(out)
        out.close()
        log.debug("Wrote kickstart %s" % ksfile)
    return True # XML-RPC requires a return value

def listkickstarts(filter={}):
    '''Return a list of kickstart names and descriptions.'''
    ksdb_dir = getoption("ksdb_dir")
    template_dir = getoption("templatedir")
    name_filter = filter.get("name","")
    desc_filter = filter.get("desc","")

    sys.path.append(ksdb_dir)

    # Generate a list of candidate files
    lst = map(lambda x: (os.path.splitext(os.path.basename(x)), x),
            glob.glob(ksdb_dir + "/*.ks") + glob.glob(template_dir + "/*.py"))

    ksdict = dict()
    # loop through '.ks' and '.py' kickstart templates
    for ((ksname,ext), fname) in lst:
        if ksname.find(name_filter) < 0:
            continue

        if ext in ".ks":
            k = Kickstart()
        elif ext in ".py":
            k = PyKickstart()
        else:
            # unknown file extension, ignore
            continue

        # attempt to read the kickstart file
        try:
            k.read(fname)
            if k.description.find(desc_filter) < 0:
                continue
            ksdict[ksname] = k.description
        except Exception, e:
            # Something went wrong while reading that kickstart.
            #print "failed to read %s: %s" % (fname,e)
            continue

    kslist = ksdict.items()
    kslist.sort()
    return kslist

def generate(name,kwargs={}):
    '''Generate a kickstart script, using the named template, modified by
    the given arguments.
    Accepted keyword arguments:
      commands=(list): Modify the template with the given kickstart commands.
                       The argument should be a list of command lines.
      version=(string): Generate a kickstart for the given kickstart language
                        version.
    '''
    # TODO: scripts=(list), packages=(list), tree=url|id, machine=id
    # XXX where does kernel commandline come in?

    ksversion = kwargs.get("version","devel")

    ksdb_dir = getoption("ksdb_dir")
    ksfile = os.path.join(ksdb_dir, "%s.ks" % name)
    template_dir = getoption("templatedir")
    pyfile = os.path.join(template_dir, "%s.py" % name)

    # try generating the python version first
    if os.path.isfile(pyfile):
        k = PyKickstart(version=ksversion)
        k.read(pyfile, reset=False)

    # okay no python template, lets just try a .ks file
    else:
        k = Kickstart(version=ksversion)
        k.read(ksfile, reset=False)

    # process and cli provided over-rides
    if 'commands' in kwargs:
        for l in kwargs.get("commands",[]):
            k.handlecommand(l)
    return str(k)

def rename(old,new):
    '''Rename the named kickstart to another name'''
    ksdb_dir = getoption("ksdb_dir")
    txt_ks_path = os.path.join(ksdb_dir, "%s.ks" % old)
    template_dir = getoption("templatedir")
    py_ks_path = os.path.join(template_dir, "%s.py" % old)

    for oldfile in (py_ks_path, txt_ks_path):
        if not os.path.isfile(oldfile):
            continue

        (name,ext) = os.path.splitext(oldfile)
        newfile = os.path.join(os.path.dirname(oldfile),"%s%s" % (new, ext))

        # already a kickstart with the matching name?
        if os.path.isfile(newfile):
            raise Exception, "Kickstart file named '%s' already exists" % new

        os.rename(oldfile,newfile)
        log.debug("Kickstart renamed: %s -> %s" % (oldfile, newfile))

    return True # XML-RPC requires a return value

def describe(name,desc):
    '''Update description for the named kickstart'''
    ksdb_dir = getoption("ksdb_dir")
    txt_ksfile = os.path.join(ksdb_dir, "%s.ks" % name)
    template_dir = getoption("templatedir")
    py_ksfile = os.path.join(template_dir, "%s.py" % name)

    if os.path.isfile(py_ksfile):
        raise Exception, "Describing PyKickstart templates not supported"
    elif os.path.isfile(txt_ksfile):
        # Read existing data (removing any description)
        fp = open(ksfile, 'r')
        data = fp.read()
        fp.close()

        # Write kickstart file using new data
        write_data(data,name,desc)
        fp.close()
    else:
        raise Exception, "Kickstart file '%s' not found" % name
    return True # XML-RPC requires a return value

def remove(ks):
    '''Remove the named kickstart from the database'''
    ksdb_dir = getoption("ksdb_dir")
    txt_ks_path = os.path.join(ksdb_dir, "%s.ks" % ks)
    template_dir = getoption("templatedir")
    py_ks_path = os.path.join(template_dir, "%s.py" % ks)

    # look for python (.py) template first, then a standard text template (.ks)
    for ksfile in (py_ks_path, txt_ks_path):
        if os.path.isfile(ksfile):
            os.unlink(ksfile)
            log.debug("Removed '%s'" % ksfile)
            if ksfile.endswith(".py") and os.path.isfile(ksfile + 'c'):
                '''remove the pre-compiled python .pyc'''
                os.unlink(ksfile + 'c')
                log.debug("Removed '%sc'" % ksfile)
            break
    # FIXME - fail if no such 'ks' found? --- or will the traceback handle that
    return True # XML-RPC requires a return value
